/* Copyright 1999 by Herve Regad-Pellagru, E-mail: herve.regadpellagru@free.fr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>

#include <x10_api.h>

#include "proto.h"
#include "common.h"


int accept_socket=0,sock=0;
int listen_mode;
struct x10_status_t *x10_status=NULL;
FILE *log_file;

static void usage(void)
{
  fprintf(stderr,"Usage: x10_server device port\n");
  exit(1);
}


/* Here, we just setup the listening connection and do it NON_BLOCK as well
   to be able to listen on the CM11A, in the meantime */
static int init_server(int port)
{
  struct sockaddr_in server;
  int s;

  memset((char *)&server,0,sizeof(server));
  server.sin_family=AF_INET;
  server.sin_port=htons((unsigned short)port);
  server.sin_addr.s_addr=INADDR_ANY;

  if ((s = socket(AF_INET,SOCK_STREAM,IPPROTO_TCP)) == -1)
    fatal_error("socket returned is invalid !!\n");

  if((fcntl(s,F_SETFL,O_NONBLOCK)))
    fatal_error("fcntl O_NONBLOCK failed !!\n");

  if (bind(s,(struct sockaddr *) &server,sizeof(server)) == -1)
    fatal_error("bind failed !!\n");

  if (listen(s,128) == -1)
    fatal_error("listen failed !!\n");

  return s;
}


/* This is the error handler, as passed to x10_open(). We just log the event */
static void error_handler(int code,char *arg,struct x10_status_t *status)
{
  char string[MAXBUF];

  sprintf(string,"Erreur: %s %s",err2string(code),arg);
  server_log_event(string);
}


/* This is the callback function called by the x10_lib layer when any X10 event
   is seen on the AC bus. We refresh the status of the module and log the event
   in the server log. The nice trick is we just have, when the listen_mode flag
   is on, to send the event to our client. That's is, we've done our spy client ! */
static void function_hook(unsigned char house,int unit,int function,
		   int dim_level,struct x10_status_t *status)
{
  char string[MAXBUF];

  x10_log_event(house,unit,function,status);
  sprintf(string,"** Function %s (%d) on unit %c%d **\n",cmd2string(function),dim_level,house,unit);
  server_log_event(string);

  /* is not in listen mode, return */
  if(!listen_mode) return;

  SEND_STRING(string);
  SEND_STRING("OK\n");
}


/* When receiving the TERM signal, we close everything and exit */
static void TERM_handler(int signal)
{
  if(sock)
    close(sock);

  if(accept_socket)
    close(accept_socket);

  if(x10_status)
    x10_close(x10_status);

  if(log_file)
    fclose(log_file);

  exit(0);
}


int main(int argc,char **argv)
{
  sigset_t signals;

  if(argc != 3)
    usage();

  /* We open our log file */
  if(!(log_file = fopen("x10.log","a+")))
    fatal_error("unable to open log file !!\n");

  /* We open the X10lig layer */
  if(!(x10_status = x10_open(argv[1],error_handler,function_hook,TIMEOUT_READ_CM11A)))
    fatal_error("unable to open serial port !!\n");

  /* First we init the listening socket */
  accept_socket = init_server(atoi(argv[2]));
  /* We then add the cleanup and exit handler for SIGTERM and set
     a set of signals to block during CM11A chat, to avoid nasty problems
     with the already nasty hardware :-) */
  sigemptyset(&signals);
  sigaddset(&signals,SIGTERM);
  signal(SIGTERM,TERM_handler);
  /* SIGPIPE will be ignored as in case of peer dying unexpectedly, we'll see it
     through read(2) returning 0 */
  signal(SIGPIPE,SIG_IGN);

  while(1)
    {
      /* We cycle on new connections or timeout */
      cycle_on_connections(accept_socket);

      /* We read news from the CM11A and block signals processing
	 in the meantime */
      sigprocmask(SIG_BLOCK,&signals,NULL);
      x10_listen_to_events(x10_status,PERIOD_POLL_CM11A);
      sigprocmask(SIG_UNBLOCK,&signals,NULL);
    }

  return(0);
}
